from django import forms
from django.forms.utils import ErrorList
from mptt.forms import TreeNodeChoiceField

from simple_locations.models import Area, AreaType


class LocationForm(forms.Form):
    name = forms.CharField(max_length=100)
    code = forms.CharField(max_length=50, required=False)
    pk = forms.CharField(widget=forms.HiddenInput(), required=False)
    target = TreeNodeChoiceField(queryset=Area.objects.all(), level_indicator="++", required=False)
    lat = forms.DecimalField(required=False)
    lon = forms.DecimalField(required=False)
    kind = forms.ModelChoiceField(
        required=False,
        empty_label="-----",
        queryset=AreaType.objects.all(),
        to_field_name="name",
    )
    move_choice = forms.BooleanField(required=False)
    position = forms.ChoiceField(
        choices=(("last-child", "inside"), ("left", "before"), ("right", "after")),
        required=False,
    )

    def clean(self):
        """make sure that both lat and lon are provided. if lat is given then lon is also required and vice versa."""
        lat = self.cleaned_data["lat"]
        lon = self.cleaned_data["lon"]
        if not lat and lon:
            msg = "Please provide the latitude"
            self._errors["lat"] = ErrorList([msg])
            return ""
        elif lat and not lon:
            msg = "Please provide the longitude"
            self._errors["lon"] = ErrorList([msg])
            return ""
        if lat and lon:
            if not -90 <= lat <= 90:
                msg = "Invalid latitude must be between 90 and -90"
                self._errors["lat"] = ErrorList([msg])
                return ""
            if not -180 <= lon <= 180:
                msg = "Invalid latitude must be between 180 and -180"
                self._errors["lon"] = ErrorList([msg])
                return ""

        return self.cleaned_data



import io
from django.db import models
from django.contrib.gis.db.models.functions import AsGeoJSON
from django.http import HttpResponse


class Quantize(models.Func):
    """
    ST_QuantizeCoordinates determines the number of bits (N)
    required to represent a coordinate value with a specified number
    of digits after the decimal point, and then sets all but the N
    most significant bits to zero. The resulting coordinate value will
    still round to the original value, but will have improved compressiblity
    """

    function = "ST_QuantizeCoordinates"
    template = "%(function)s(%(expressions)s, %(quantize)s)"


class Simplify(models.Func):
    """
    Returns a "simplified" version of the given geometry using the Douglas-Peucker algorithm.
    """

    function = "ST_SIMPLIFY"
    template = "%(function)s(%(expressions)s, %(simplify)s)"


class Multi(models.Func):
    """
    Returns the geometry as a MULTI* geometry collection.
    If the geometry is already a collection, it is returned unchanged.
    """

    function = "ST_Multi"


class AreaQueryset(models.QuerySet):
    def _annotate_geometries(self, simplify: float = 1e-3, quantize: int = 5) -> models.QuerySet:
        return self.annotate(
            geojson=AsGeoJSON(
                Multi(
                    Quantize(
                        Simplify(
                            "geom",
                            simplify=simplify,
                        ),
                        quantize=quantize,
                    )
                )
            )
        )

    def to_geojson(self, simplify: float = 1e-3, quantize: int = 5) -> str:
        """
        Intended to be a highly performant geoJSON generator, this is inspired by Django's
        geojson serializer but simplified, and carries out more operations in the DB including
        geometry simplifying and quantization

        This returns a string, in order to prevent multiple ser / de-ser calls
        """

        output = io.StringIO()  # Avoids a round-trip deserializer by using a raw StringIO
        prefix = """{"type": "FeatureCollection", "crs": {"type": "name", "properties": {"name": "EPSG:4326"}}, "features": ["""
        pattern = '{"type": "Feature", "geometry": %s, "properties": {"id": %s } }'
        suffix = "]}"

        output.write(prefix)
        is_first = True
        for obj in self.filter(geom__isnull=False)._annotate_geometries(simplify=simplify, quantize=quantize):
            if is_first:
                is_first = False
            else:
                output.write(",")
            output.write(pattern % (obj.geojson, obj.id))

        output.write(suffix)

        return output.getvalue()

    def to_response(self):
        """
        Directly return a valid HTTPResponse
        """
        return HttpResponse(content_type="application/json", content=self.to_geojson())



from datetime import date
from decimal import Decimal
from typing import Any, Optional

from django.db import models
from mptt.models import MPTTModel

class DateStampedModel(models.Model):
    date_created: date
    date_modified: date
    class Meta:
        abstract: bool

class Point(models.Model):
    class Meta:
        verbose_name: str
        verbose_name_plural: str
        app_label: str
    latitude: Decimal
    longitude: Decimal

class AreaType(models.Model):
    class Meta:
        verbose_name: str
        verbose_name_plural: str
        app_label: str
    name: Any
    slug: Any

class Area(MPTTModel):
    class Meta:
        unique_together: bool
        verbose_name: str
        verbose_name_plural: str
        app_label: str
    class MPTTMeta:
        parent_attr: str
        order_insertion_by: Any
    name: str
    code: str
    kind: AreaType
    kind_id: Optional[int]
    location: Optional[Point]
    location_id: Optional[int]
    geom: Any
    parent: Area
    parent_id: int
    def delete(self) -> None: ...
    def get_ancestor_at_level(self, level: int = ...) -> Area: ...
    def display_name_and_type(self): ...
    def display_with_parent(self): ...

class AreaProfile(DateStampedModel):
    area: Any
    description: Any

class IndicatorMeasureSchema(models.Model):
    schema: Any

class AreaIndicator(DateStampedModel):
    class IndicatorMeasureChoice(models.TextChoices):
        UNIT: Any
        PERCENTAGE: Any
        NOMINAL: Any
        ORDINAL: Any
        QUALITATIVE: Any
    area: Any
    name: Any
    measure: Any
    value: Any
    dimensions: Any



import os
import shutil
import tempfile
import urllib
import zipfile
from pathlib import Path
from typing import Any, List, NamedTuple
from warnings import warn

from django.contrib.gis.db.models import Union
from django.contrib.gis.gdal import DataSource  # type: ignore
from django.contrib.gis.geos import GEOSGeometry, MultiPolygon
from django.core.management.base import BaseCommand
from django.db import IntegrityError

from simple_locations.models import Area, AreaType


class RenameDistrict(NamedTuple):
    nso_name: str
    wiki_name: str


area_definition_set = {
    "png_llg_boundaries_2011census_region.shp": {
        "areatype": "llg",
        "name": "LLGNAME",
        "code": "GEOCODE",
    },
    "png_dist_boundaries_2011census_region.shp": {
        "areatype": "district",
        "name": "DISTNAME",
        "code": "GEOCODE",
    },
    "png_prov_boundaries_2011census_region.shp": {
        "areatype": "province",
        "name": "PROVNAME",
        "code": "PROVID",
    },
}


class Command(BaseCommand):

    help = """Import data from NSO PNG Boundaries zip file"""

    def handle(self, *args: Any, **options: Any):
        self.import_zip()
        # self.import_directory('/home/josh/Downloads/NSO_PNG Boundaries')
        self.merge_districts()
        self.rebuild_tree()
        self.perform_rename()

    def import_zip(
        self,
        zip_url: str = "https://png-data.sprep.org/system/files/NSO_PNG%20Boundaries.zip",
    ):

        with urllib.request.urlopen(zip_url) as response, tempfile.NamedTemporaryFile() as tmp_file:  # type: ignore
            shutil.copyfileobj(response, tmp_file)
            with zipfile.ZipFile(tmp_file) as tmpzip:
                with tempfile.TemporaryDirectory() as tmpdirname:
                    tmpzip.extractall(tmpdirname)
                    self.import_directory(tmpdirname)

    def import_directory(self, tmpdirname: str):
        self.stderr.write(self.style.NOTICE("Importing shapes"))
        for filename in [f for f in os.listdir(tmpdirname) if f.endswith(".shp")]:

            area_definition = area_definition_set[filename]
            area_type = AreaType.objects.get_or_create(
                name=area_definition["areatype"],
                slug=area_definition["areatype"],
            )[
                0
            ]  # type: AreaType
            self.stderr.write(self.style.NOTICE(f"Importing {tmpdirname} / {filename}"))
            self.import_shp(
                shape_path=Path(tmpdirname) / filename,
                area_type=area_type,
                field_mapping=area_definition,
            )

    def import_shp(self, shape_path: Path, area_type: AreaType, field_mapping: dict) -> None:

        for feature in DataSource(str(shape_path))[0]:
            self.import_feature(feature=feature, area_type=area_type, field_mapping=field_mapping)

    def import_feature(
        self,
        feature,
        area_type: AreaType,
        field_mapping: dict,
        database_srid: int = 4326,
    ):
        # Transform the geometry to a MultiPolygon if it is a Single Polygon
        # This is because a Shapefile may contain a data type which we don't cover (like a Polygon)
        geom = feature.geom.geos if isinstance(feature.geom.geos, MultiPolygon) else MultiPolygon(feature.geom.geos)
        code = str(feature.get(field_mapping["code"]))
        geometry_srid = feature.geom.srid

        # Transform the geometry if required
        # to suit the database SRID
        asset_geometry = GEOSGeometry(geom, srid=geometry_srid)
        if geometry_srid != database_srid:
            asset_geometry.transform(database_srid)

        if Area.objects.filter(code=code).exists():
            self.stderr.write(self.style.NOTICE(f"Skip {code} because it already exists"))
            return Area.objects.get(code=code)
        try:
            area = Area.objects.create(
                name=feature.get(field_mapping["name"]),
                code=code,
                kind=area_type,
                geom=asset_geometry,
            )
            return area
        except IntegrityError as E:
            self.stderr.write(self.style.ERROR(f"Error writing code {code}: {E}"[:140] + "..."))

    def merge_districts(self):
        """
        As we do not have a unified "Country" shape, merge the Districts to create the top level
        """
        self.stderr.write(self.style.SUCCESS("Merge provinces"))
        country_level_area = Area.objects.get_or_create(
            kind=AreaType.objects.get_or_create(name="country", slug="country")[0],
            geom=Area.objects.filter(kind__name="province").aggregate(Union("geom"))[
                "geom__union"
            ],  # Multipolygon object
            name="Papua New Guinea",
            code="PNG",
        )[0]

        for d in Area.objects.filter(kind__name="province"):
            d.parent = country_level_area
            d.save()

    def rebuild_tree(self):

        self.stderr.write(self.style.SUCCESS("Reset Area parent code"))
        for a in Area.objects.exclude(kind__name__in=["country", "province"]):
            try:
                a.parent = Area.objects.get(code=a.code[:-2])
                a.save()
            except Exception as E:
                warn(f"{E}")
                pass

        self.stderr.write(self.style.SUCCESS("Rebuild the locations Area tree"))
        Area.objects.rebuild()

    def perform_rename(self):

        sources = """Abau District	Abau District
        Aitape-Lumi District	Aitape/Lumi District
        Alotau District	Alotau District
        Ambunti-Dreikikier District	Ambunti/Drekikier District
        Anglimp-South Waghi District	Anglimp/South Waghi District
        Angoram District	Angoram District
        Bogia District	Bogia District
        Bulolo_District	Bulolo District
        Central Bougainville District	Central Bougainville District
        Chuave District	Chuave District
        Daulo District	Daulo District
        Dei District	Dei District
        Esa'ala District	Esa'ala District
        Finschhafen District	Finschafen District
        Gazelle District	Gazelle District
        Goilala District	Goilala District
        Goroka District	Goroka District
        Gumine District	Gumine District
        Henganofi District	Henganofi District
        Huon District	Huon District
        Ialibu-Pangia District	Ialibu/Pangia District
        Ijivitari District	Ijivitari District
        Imbonggu District	Imbonggu District
        Jimi District	Jimi District
        Kabwum District	Kabwum District
        Kagua-Erave District	Kagua/Erave District
        Kainantu District	Kainanatu District
        Kairuku-Hiri District	Kairuku - Hiri District
        Kandep District	Kandep District
        Kandrian-Gloucester District	Kandrian/Gloucester District
        Karimui-Nomane District	Karimui/Nomane District
        Kavieng District	Kavieng District
        Kerema District	Kerema District
        Kerowagi District	Kerowagi District
        Kikori District	Kikori District
        Kiriwini-Goodenough District	Kiriwina-Goodenough District
        Kokopo District	Kokopo District
        Komo-Magarima District	Komo/Magarima District
        Kompiam Ambum District	Kompiam District
        Koroba-Kopiago District	Koroba/Kopiago District
        Kundiawa-Gembogl District	Kundiawa/Gembogl District
        Lae District	Lae District
        Lagaip-Porgera District	Lagaip/Pogera District
        Lufa District	Lufa District
        Madang District	Madang District
        Manus District	Manus District
        Maprik District	Maprik District
        Markham District	Markham District
        Mendi-Munihu District	Mendi/Munihu District
        Menyamya District	Menyamya District
        Middle Fly District	Middle Fly District
        Middle Ramu District	Middle Ramu District
        Mount Hagen District	Mt Hagen District
        Mul-Baiyer District	Mul/Baiyer District
        Namatanai District	Namatanai District
        National Capital District	National Capital District
        Nawae District	Nawae District
        Nipa-Kutubu District	Nipa/Kutubu District
        North Bougainville District	North Bougainville District
        North Fly District	North Fly District
        North Waghi District	North Waghi District
        Nuku District	Nuku District
        Obura-Wonenara District	Obura/Wonenara District
        Okapa District	Okapa District
        Pomio District	Pomio District
        Rabaul District	Rabaul District
        Rai Coast District	Rai Coast District
        Rigo District	Rigo District
        Samarai-Murua District	Samarai-Murua District
        Sina Sina-Yonggomugl District	Sina Sina Yonggomugl District
        Sohe District	Sohe District
        South Bougainville District	South Bougainville District
        South Fly District	South Fly District
        Sumkar District	Sumkar District
        Talasea District	Talasea District
        Tambul-Nebilyer District	Tambul/Nebilyer District
        Tari-Pori District	Tari/Pori District
        Tawae-Siassi District	Tawae/Siassi District
        Telefomin District	Telefomin District
        Unggai-Benna District	Unggai/Benna District
        Usino Bundi District	Usino Bundi District
        Vanimo-Green River District	Vanimo/Green River District
        Wabag District	Wabag District
        Wapenamanda District	Wapenamanda District
        Wewak District	Wewak District
        Wosera-Gawi District	Wosera Gawi District
        Yangoru-Saussia District	Yangoru Saussia District"""

        renames = []  # type: List[RenameDistrict]

        for s in sources.split("\n"):
            n = s.split("\t")
            renames.append(RenameDistrict(n[1].strip(), n[0].strip()))

        # Area.objects.all().delete()
        self.stdout.write(self.style.MIGRATE_HEADING("~~~ Starting rename ~~~"))
        for rename in renames:
            try:
                area = Area.objects.get(name=rename.nso_name, kind__name="district")
            except Area.DoesNotExist:
                self.stdout.write(self.style.WARNING(f'Area does not exist: "{rename.nso_name}"'))
                continue

            if area.name == rename.wiki_name:
                self.stdout.write(self.style.SUCCESS(f'No change: "{rename.nso_name}"'))
            else:
                area.name = rename.wiki_name
                area.save()
                self.stdout.write(
                    self.style.SUCCESS(f'Successfully renamed: "{rename.nso_name}" to "{rename.wiki_name}s')
                )



from typing import Iterable, List, Optional, Type

from django.contrib.gis.db.models import (
    GeometryField,
    LineStringField,
    MultiPolygonField,
)
from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy as __
from mptt.models import MPTTModel

from simple_locations.manager import AreaQueryset


def get_geom_field(model) -> GeometryField:
    """
    Returns the first field likely to be a geometry field
    from a model
    """
    fields = model._meta.fields  # type: List
    for field_ in fields:
        if isinstance(field_, GeometryField):
            return field_
    raise KeyError(f"No geometry field could be identified for {model}")


def intersects_areas(area_ids: Iterable[int], model: models.Model, geom_field: Optional[str] = None):
    """
    Return parameters for an 'extra' clause intersecting an area
    This function uses SRID 3857 and the corresponding projected areas
    for compatibility with osm import

    This example - from DIRD's location profile airstrips:

    >>> extra_clause = intersects_areas(Area.objects.values_list('id', flat=True), Airstrip)
    >>> Airstrip.objects.extra(**extra_clause, Airstrip)

    """

    def _area_model(srid: int) -> Type[models.Model]:
        """
        Determine which model to apply the intersection to
        based on the SRID
        """
        if srid == 4326:
            return Area
        elif srid == 3857:
            return ProjectedArea
        raise AssertionError("Unhandled SRID")

    geom_field_instance = model._meta.get_field(geom_field) if geom_field else get_geom_field(model)

    if not isinstance(geom_field_instance, GeometryField):
        raise TypeError

    geom_field_name: str = geom_field_instance.db_column or geom_field_instance.attname

    area_query_values = ",".join(map(str, area_ids))
    area_clause = f"""ANY ('{{{area_query_values}}}'::int[])"""

    area_model = _area_model(geom_field_instance.srid)
    area_table_name = area_model._meta.db_table
    area_table_pk = area_model._meta.pk.db_column or area_model._meta.pk.attname

    return dict(
        tables=[area_table_name],
        where=[
            f'"{area_table_name}"."{area_table_pk}" = {area_clause}',
            f'ST_INTERSECTS("{area_table_name}"."geom", "{model._meta.db_table}"."{geom_field_name}")',
        ],
    )


class DateStampedModel(models.Model):
    date_created = models.DateField(verbose_name=_("Date Created"), auto_now_add=True, null=True, blank=True)
    date_modified = models.DateField(verbose_name=_("Last Modified"), null=True, blank=True)

    class Meta:
        abstract = True


class Point(models.Model):
    class Meta:
        verbose_name = __("Point")
        verbose_name_plural = __("Points")
        app_label = "simple_locations"

    latitude = models.DecimalField(max_digits=13, decimal_places=10)
    longitude = models.DecimalField(max_digits=13, decimal_places=10)

    def __str__(self):
        return _("%(lat)s, %(lon)s") % {"lat": self.latitude, "lon": self.longitude}


class AreaType(models.Model):
    class Meta:
        verbose_name = __("Area Type")
        verbose_name_plural = __("Area Types")
        app_label = "simple_locations"

    name = models.CharField(max_length=100)
    slug = models.CharField(max_length=30, unique=True)

    def __str__(self):
        return _(self.name)


class Area(MPTTModel):
    class Meta:
        unique_together = ("code", "kind")
        verbose_name = __("Area")
        verbose_name_plural = __("Areas")
        app_label = "simple_locations"

    class MPTTMeta:
        parent_attr = "parent"
        order_insertion_by = ["name"]

    name = models.CharField(max_length=100)
    code = models.CharField(max_length=50, unique=True)  # was CodeField
    kind = models.ForeignKey("AreaType", blank=True, null=True, on_delete=models.CASCADE)
    location = models.ForeignKey(Point, blank=True, null=True, on_delete=models.CASCADE)
    geom = MultiPolygonField(srid=4326, blank=True, null=True)
    parent = models.ForeignKey("self", blank=True, null=True, related_name="children", on_delete=models.CASCADE)

    def delete(self):
        super(Area, self).delete()

    def get_ancestor_at_level(self, level=2) -> "Area":
        """Get the area ancestor at a given level

        Will travel the tree until it reaches the level or return self if already under that level"""
        if self.get_level() <= level:
            return self
        return self.get_ancestors()[level]

    def display_name_and_type(self) -> str:
        """Area name and type

        Example District of Bamako"""
        return f"{self.kind.name} of {self.name}"

    def display_with_parent(self) -> str:
        """Print Area name and kind and parent name and kind

        Example: Aldeia of Baha-Neo in Suco of Lia Ruca"""
        if not self.parent:
            return self.display_name_and_type()
        elif self.kind.name == "District":
            return self.display_name_and_type()
        else:
            return "%(this)s in %(parent)s" % {
                "this": self.display_name_and_type(),
                "parent": self.parent.display_name_and_type(),
            }

    def __str__(self) -> str:
        """print Area name from its Kind and parent

        Example: Bamako"""

        # don't add-in kind if kind name is already part of name.
        # if (not self.parent) or (not self.kind) or self.name.startswith(self.kind.name):
        #    return self.name
        # else:
        #    return _(u"%(type)s of %(area)s.") % {'type': self.kind.name, \
        #                                              'area': self.name,}

        return self.name

    geofunctions = AreaQueryset.as_manager()


class ProjectedArea(models.Model):
    """
    Projected "area" instances in the common web mercator (3857)
    This allows for correctly indexed spatial queries against data which is
    in that coordinates system when ingested.
    Most commonly this would be OSM data
    """

    geom = MultiPolygonField(null=True, blank=True, srid=3857)
    area = models.OneToOneField("Area", primary_key=True, on_delete=models.CASCADE)


class Border(models.Model):
    """
    Shared parts of border topologies are referenced
    here in order to make a more efficient mapping layer.
    When we do this we can greatly reduce the amount of data
    sent to client (for PNG 'area' is 9.6M on-disk, 'lines' is 2.3M on-disk)
    """

    # srid could be 4326 or 3857. 3857 is easier for simplification
    # because it's in meters; simplification in degrees is not fun.
    geom = LineStringField(srid=3857)
    area = models.ManyToManyField("Area")

    # The following fields are denormalised in order to
    # simplify generting and filtering vector data
    area_ids = ArrayField(models.IntegerField(), default=list)
    area_types = ArrayField(models.IntegerField(), default=list)


class AreaProfile(DateStampedModel):
    area = models.OneToOneField("Area", on_delete=models.CASCADE, primary_key=True)
    description = models.TextField()


class IndicatorMeasureSchema(models.Model):
    """
    Declare the "shape" of the data to be displayed.
    See :
    https://pypi.org/project/jsonschema/
    https://json-schema.org/
    """

    schema = models.TextField()


class AreaIndicator(DateStampedModel):
    """
    Initially required for PNG DIMS
    """

    class IndicatorMeasureChoice(models.TextChoices):
        """
        Derived from the IATI standard, this determines the "type" of
        data which is being measured - is it percentage, count, or on some sort of scale?
        """

        UNIT = "U", _("Units")
        PERCENTAGE = "P", _("Percentages")
        NOMINAL = "N", _("Nominal")
        ORDINAL = "O", _("Ordinal")
        QUALITATIVE = "Q", _("Qualitative")

    area = models.ForeignKey("Area", on_delete=models.CASCADE)
    name = models.TextField()

    measure = models.CharField(
        max_length=2,
        choices=IndicatorMeasureChoice.choices,
        default=IndicatorMeasureChoice.UNIT,
        help_text=_("Define the unit of measure in which the value is reported."),
    )

    # In order to permit for instance a time-series-keyed mesaurement,
    # this is a JSON field rather than a text field
    value = models.JSONField()

    # Define parameters for the measurement.
    # For example: {Gender: "Male", age_range: [0,16], is_smoker: no}
    dimensions = models.JSONField(help_text=_("A category used for disaggregating the result by gender, age, etc."))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2017-04-03 06:07
from __future__ import unicode_literals

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0004_auto_20170330_1618'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='area',
            managers=[
                ('tree', django.db.models.manager.Manager()),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-03-05 08:26
from __future__ import unicode_literals

import django.contrib.gis.db.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0006_auto_20180202_1540'),
    ]

    operations = [
        migrations.AlterField(
            model_name='area',
            name='geom',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(blank=True, null=True, srid=4326),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-02-02 06:40
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0005_auto_20170403_1507'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='area',
            managers=[
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['simple_locations',
 'simple_locations.management',
 'simple_locations.management.commands',
 'simple_locations.migrations']

package_data = \
{'': ['*'],
 'simple_locations': ['fixtures/*',
                      'locale/fr/LC_MESSAGES/*',
                      'static/css/*',
                      'static/images/*',
                      'static/javascripts/*',
                      'static/uni_form/*',
                      'templates/simple_locations/*',
                      'templates/simple_locations/admin/*']}

install_requires = \
['django-geojson>=3.2.1,<4.0.0', 'django-mptt>=0.13.4,<0.14.0']

setup_kwargs = {
    'name': 'simple-locations',
    'version': '3.1.7',
    'description': "The common location package for Catalpa's projects",
    'long_description': '# simple_locations\n\nThe common location package used for catalpa\'s projects. A hierarchical tree of geographical locations supporting location type and GIS data.\n\n## Admin\n\nThe admin site is set up to use Modeltranslations (if available in the parent app)\n\nFor modeltranslations, please remember to run `sync_translation_fields` in order to get `name_en`, `name_tet` etc. fields.\n\n## Environment\n\nThis is intended to be compatible with:\n\n- Django 3.1, 3.2, 4.0\n- Python 3.7, 3.8, 3.9\n\n```sh\ngh repo clone catalpainternational/simple_locations\ncd simple_locations\npython -m venv env\n. env/bin/activate\npip install pip-tools\npip-sync requirements.txt dev.txt\npre-commit install\n```\n\n### Pre Commit\n\nIf `pre-commit` is installed your code will be checked before commit.\nThis includes\n\n- black\n- flake8\n- isort\n- mypy\n\nThe same checks are run on push. See `pytest.yaml` for details on the checks being run.\n\n### New Release\n\nFor a new release, change the `version` property in pyproject.toml and push a git tag with the version number\nFor instance at time of writing the version is `3.1.4` with the tag `v3.1.4`\n\nSee `build.yaml` for details on release tagging\n## Changelog\n\n- Version 3.1.4\n  - Migrating JSON views from openly\n\n- Version 3.1.3\n  - Added `intersects_area` function\n\n- Version 3.1.2\n  - Uses psycopg2-binary for development environment\n\n- Version 3.1.1\n  - Added "border" fields\n  - Added a model for "projected" areas in EPSG:3857\n  - Added commands for border generation and\n  - `./manage.py` and associated project code\n\n- Version 3.0.1\n\n  - Poetry for dependency + packaging\n  - Releases are automated by pushing a `vx.x.x` tag to github\n\n- Version 3.0 (not on pypi)\n\n  - Code style changes (black, flake8)\n\n- Version 2.77\n\n  - first pass of updates for Python 3.8+ and Django 3.1+\n\n- Version 2.75\n\n  - add modeltranslations\n\n- Version 2.74\n\n  - fix CORS issue breaking maps in AreaAdmin\n  - typo in AreaChildrenInline\n\n- Version 2.73\n\n  - add an inline showing children to the Area admin\n  - make the `geom` field optional\n\n- Version 2.72\n  - optionally use django_extensions\' ForeignKeyAutocompleteAdmin in admin interface\n\n\n## Manually Uploading a new version to PyPi\n\nBump `pyproject.toml`\nThen run `poetry build` and `poetry publish`\n\n```bash\npoetry build\npoetry publish\n```\n\nSee the file `build.yml` for the workflow\n',
    'author': 'Joshua Brooks',
    'author_email': 'josh@catalpa.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/catalpainternational/simple_locations',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Generated by Django 3.1.13 on 2022-02-20 16:03

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("simple_locations", "0010_add_border_model"),
    ]

    operations = [
        migrations.AddField(
            model_name="border",
            name="area_ids",
            field=django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), default=list, size=None),
        ),
        migrations.AddField(
            model_name="border",
            name="area_types",
            field=django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), default=list, size=None),
        ),
    ]



# Generated by Django 3.1.14 on 2022-03-07 06:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0012_auto_20220221_0817'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='projectedarea',
            name='id',
        ),
        migrations.AlterField(
            model_name='area',
            name='level',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='area',
            name='lft',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='area',
            name='rght',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='projectedarea',
            name='area',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='simple_locations.area'),
        ),
    ]



# Generated by Django 3.1.13 on 2021-11-11 08:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('simple_locations', '0008_auto_20200804_0554'),
    ]

    operations = [
        migrations.AlterField(
            model_name='area',
            name='code',
            field=models.CharField(max_length=50, unique=True),
        )
    ]
